package java.security;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.io.ObjectOutputStream;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.io.ObjectStreamField;
import java.util.Map;
import java.io.Serializable;

final class BasicPermissionCollection extends PermissionCollection implements Serializable
{
    private static final long serialVersionUID = 739301742472979399L;
    private transient Map<String, Permission> perms;
    private boolean all_allowed;
    private Class<?> permClass;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public BasicPermissionCollection(Class<?> permClass) {
        this.perms = new HashMap(11);
        this.all_allowed = false;
        this.permClass = permClass;
    }
    
    @Override
    public void add(Permission permission) {
        if (!(permission instanceof BasicPermission)) {
            throw new IllegalArgumentException("invalid permission: " + permission);
        }
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
        }
        BasicPermission basicPermission = (BasicPermission)permission;
        if (this.permClass == null) {
            this.permClass = basicPermission.getClass();
        }
        else if (basicPermission.getClass() != this.permClass) {
            throw new IllegalArgumentException("invalid permission: " + permission);
        }
        synchronized (this) {
            this.perms.put(basicPermission.getCanonicalName(), permission);
        }
        if (!this.all_allowed && basicPermission.getCanonicalName().equals("*")) {
            this.all_allowed = true;
        }
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof BasicPermission)) {
            return false;
        }
        BasicPermission basicPermission = (BasicPermission)permission;
        if (basicPermission.getClass() != this.permClass) {
            return false;
        }
        if (this.all_allowed) {
            return true;
        }
        String s = basicPermission.getCanonicalName();
        Permission permission2;
        synchronized (this) {
            permission2 = (Permission)this.perms.get(s);
        }
        if (permission2 != null) {
            return permission2.implies(permission);
        }
        int fromIndex = s.length() - 1;
        int lastIndex;
        while ((lastIndex = s.lastIndexOf(".", fromIndex)) != -1) {
            s = s.substring(0, lastIndex + 1) + "*";
            synchronized (this) {
                permission2 = (Permission)this.perms.get(s);
            }
            if (permission2 != null) {
                return permission2.implies(permission);
            }
            fromIndex = lastIndex - 1;
        }
        return false;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.perms.values());
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<Object, Object> hashtable = new Hashtable(this.perms.size() * 2);
        synchronized (this) {
            hashtable.putAll(this.perms);
        }
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("all_allowed", this.all_allowed);
        putFields.put("permissions", hashtable);
        putFields.put("permClass", this.permClass);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        Hashtable<Object, Object> hashtable = (Hashtable)fields.get("permissions", null);
        (this.perms = new HashMap(hashtable.size() * 2)).putAll(hashtable);
        this.all_allowed = fields.get("all_allowed", false);
        this.permClass = (Class)fields.get("permClass", null);
        if (this.permClass == null) {
            Enumeration<Object> elements = hashtable.elements();
            if (elements.hasMoreElements()) {
                this.permClass = ((Permission)elements.nextElement()).getClass();
            }
        }
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Hashtable.class), new ObjectStreamField("all_allowed", Boolean.TYPE), new ObjectStreamField("permClass", Class.class) };
    }
}
